//*****************************************************************************
// File Name            : camera_ap.v
//-----------------------------------------------------------------------------
// Function             : camera sample application for SDRAM
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2010/10/28
// -.-- 2010/11/17
// -.-- 2011/2/28
// -.-- 2011/09/08
// -.-- 2011/11/19
// -.-- 2011/12/26
//*****************************************************************************
module camera_ap ( clk_50m, rstb, 
                   led_0,led_1,led_2,led_3,
                   led_4,led_5,led_6,led_7,led_8,led_9,
                   dmy_camera_mode,
                   init_n,stop_n,
                   scl,sda,
                   txd,rxd,
                   xclk,i_pclk,i_c_vsync,i_href,i_in_data,
                   mon,
                   vga_clk,da_r,da_g,da_b,vsync,hsync,
                   ram_clk,ram_clke,ram_cs_n,ram_cas_n,    
                   ram_ras_n,ram_we_n,ram_dqml,ram_dqmu,ram_ba0,    
                   ram_ba1,ram_adr,ram_dq_l,ram_dq_u,out_en,
                   mon_data
);
  input clk_50m;    //clk 100MHz
  input rstb;   //reset_b
  output led_0; //led control 1:on 0:off
  output led_1; //led control 1:on 0:off
  output led_2; //led control 1:on 0:off
  output led_3; //led control 1:on 0:off
  output led_4; //led control 1:on 0:off
  output led_5; //led control 1:on 0:off
  output led_6; //led control 1:on 0:off
  output led_7; //led control 1:on 0:off
  output led_8; //led control 1:on 0:off
  output led_9; //led control 1:on 0:off
  input  dmy_camera_mode ;// 0:camera 1:dmy
  input  init_n ;// 0:init
  input  stop_n ;// 0:stop
  output scl;   //I2C SCL
  inout  sda;   //I2C SDA
  output txd;   //serial tx data    
  input  rxd;   //serial rx data
  output xclk;  //camera system clock
  input  i_pclk;
  input  i_c_vsync;
  input  i_href;
  input  [7:0]i_in_data;
  output  mon;
  output  vga_clk;
  output [3:0]da_r;
  output [3:0]da_g;
  output [3:0]da_b;
  output vsync;
  output hsync;
  output ram_clk;
  output ram_clke;  //sdram clke
  output ram_cs_n;  //sdram cs
  output ram_cas_n; //sdram cas
  output ram_ras_n; //sdram ras
  output ram_we_n;  //sdram write enable
  output ram_dqml;  //sdram data output enable dq[7:0]
  output ram_dqmu;  //sdram data output enable dq[15:8]
  output ram_ba0;   //sdram bank address 0
  output ram_ba1;   //sdram bank address 1
  output [11:0] ram_adr; //sdram address 
  inout [7:0] ram_dq_l; //sdram data 
  inout [7:0] ram_dq_u; //sdram data 
  output      out_en; 
  //
  output [15:0] mon_data;
   
  //
  wire  clk ; //50MHz ckl
  wire  rst_in ; //reset 
  wire xclk_dmy;  
 
  reg vsync_d1;
  reg href_d1;
  reg [7:0]data_d1;
  reg pclk_d2;
  reg vsync_d2;
  reg href_d2;
  reg [7:0]data_d2;
  reg    mon; 
  reg  [27:0] clk_cnt;
  reg  [27:0] xclk_cnt;
  reg  [27:0] clk_125m_cnt;
  reg  [27:0] clk_25m_cnt;
  reg  [27:0] clk_150m_cnt;
  //svga_if
  wire in_a_da_en;
  wire [7:0]in_a_da_r;
  wire [7:0]in_a_da_g;
  wire [7:0]in_a_da_b;
  // 
  wire in_b_da_en; 
  wire [7:0]in_b_da_r;
  wire [7:0]in_b_da_g;
  wire [7:0]in_b_da_b;
  //
  wire in_c_da_en;
  wire [7:0]in_c_da_r;
  wire [7:0]in_c_da_g;
  wire [7:0]in_c_da_b; 
  //
  wire in_d_da_en; 
  wire [7:0]in_d_da_r;
  wire [7:0]in_d_da_g;
  wire [7:0]in_d_da_b;
  //wire sync;
  wire tmg_sync;
  wire h_c_en;
  wire [9:0] v_c;
  wire [9:0] h_c;
  wire [7:0]da_r_tmp;
  wire [7:0]da_g_tmp;
  wire [7:0]da_b_tmp;
  wire da_en;
  //i2c
  wire wr;
  wire rd;
  wire [6:0]adr;
  wire [31:0]wr_data; 
  wire [2:0]wr_bytes; 
  wire [3:0]wr_be; 
  wire rd_data_en;
  wire [31:0]rd_data;
  wire [2:0]rd_bytes; 
  wire [3:0]rd_be; 
  wire    scl_drv;
  wire    sda_i;
  wire    sda_o;
  //rs232c    
  wire  [7:0] tx_fifo_data;    //tx fifo data 8bit
  wire        tx_fifo_data_en; //tx fifo data enable 
  wire       tx_fifo_empty;   //tx fifo read enable 
  wire       tx_fifo_full ;   //tx fifo read enable 
  wire       tx_fifo_n_full ; //tx fifo read enable 
  wire [7:0] tx_data;        //tx data 8bit
  wire       tx_data_en;      //tx data enable 
  wire       tx_busy;         //tx busy
  wire [7:0] rx_data;        //tx data 8bit
  wire       rx_data_en;      //tx data enable 
  wire       rx_busy;         //tx busy
  //cmd gen
  wire  cmd_wr;
  wire  cmd_rd;
  wire [7:0]cmd_adr;
  wire [7:0]cmd_data;
  wire [8:0] rgb_sel;
  //msg_buf
  wire  a_n_full;
  wire  b_n_full;
  //varm
  wire  wea;
  wire [19 : 0] addra;
  wire [11 : 0] dina;
  wire [19 : 0] addrb;
  wire [11 : 0] doutb;
  //
  wire pclk;
  wire c_vsync;
  wire href;
  wire [7:0]in_data;
  //dmy
  wire dmy_pclk;
  wire dmy_c_vsync;
  wire dmy_href;
  wire [7:0]dmy_in_data;
  wire     msg_buf_mon;
  wire     clk_125m;
  wire     clk_125m_b;
  wire     clk_150m;
  wire     clk_locked;
  wire     clk_25m;
  wire     clk_10m;
  wire     clk_200m;
  reg led_0; //led control 1:on 0:off
  reg led_1; //led control 1:on 0:off
  reg led_2; //led control 1:on 0:off
  reg led_3; //led control 1:on 0:off
  reg led_4; //led control 1:on 0:off
  reg led_5; //led control 1:on 0:off
  reg led_6; //led control 1:on 0:off
  reg led_7; //led control 1:on 0:off
  reg led_8; //led control 1:on 0:off
  reg led_9; //led control 1:on 0:off
  //reg rx_control_d1;
  reg ff_rx_sop_d1;
  reg [31:0] readdata_d1;
  wire [7:0] led_tmp;
   
  wire [11:0]v_wr_vadr;   //vram write v adr
  wire [11:0]v_wr_hadr;   //vram write h adr
  wire [11:0]v_wr_data;   //vram write h adr
  wire v_wr_line_end;     //vram write line end
  wire v_rd_req;          //vram read req
  wire [11:0]v_rd_vadr;   //vram read v adr
  wire[11:0]v_rd_hadr;    //vram read h adr
   
parameter  c25m_cnt_1s =   28'd24999999; //1S clk=25MHz
parameter  c150m_cnt_1s = 28'd149999999; //1S clk=150MHz
parameter  c10m_cnt_1s =    28'd9999999; //1S clk=10MHz
parameter  led0_cnt_1s =   28'd49999999; //1S clk=50MHz
parameter  led1_cnt_1s =  28'd124999999; //1S clk=125MHz


  wire [15:0] ram_dq_r; //sdram data 
  wire [15:0] ram_dq_w; //sdram data 
  wire u_wreq;        //user write req
  wire u_wack;        //user write ack
  wire [19:0]u_wadr;   //user write address
  wire [15:0]u_wr_da; //user write data
  wire u_wr_da_en;    //user write enable
  wire u_rreq;        //user read req
  wire u_rack;        //user read ack
  wire [19:0]u_radr;   //user read address
  wire [15:0]u_rd_da; //user read  data
  wire u_rd_da_en;         //user read  enable   
   
  wire [3:0] state ; //state 
  wire    busy;
      
// system reset 
assign  rst_in = ~ rstb;
    
// Instantiate the module
/*
clk_gen instance_name (
    .CLKIN_IN(clk_50m), 
    .RST_IN(rst_in), 
    .CLKFX_OUT(clk), 
    .CLKIN_IBUFG_OUT(CLKIN_IBUFG_OUT), 
    .CLK0_OUT(CLK0_OUT), 
    .LOCKED_OUT(LOCKED_OUT)
    );
*/
//assign clk = clk_50m;
assign vga_clk = clk_50m;
assign ram_clk = clk_150m;
   
//clk_gen
 clk_gen clk_gen(
   .areset(rst_in),
   .inclk0(clk_50m),
   .c0(clk),
   //.c1(clk_125m_b),
   .c1(clk_200m),
   .c2(clk_150m),
   .c3(clk_25m),
   .c4(clk_10m),
   .locked(clk_locked)); 

assign xclk = clk_10m;
   
always @ (posedge dmy_pclk or negedge rstb )
  if (rstb==1'b0)
    begin
      vsync_d1 <= 1'b0;
      href_d1  <= 1'b0;
      data_d1  <= 8'h00;
    end
  else
    begin
      vsync_d1 <= c_vsync ;
      href_d1  <= href;
      data_d1     <= in_data;
    end

//monter signal   
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    begin
      pclk_d2 <= 1'b0; 
      vsync_d2 <= 1'b0;
      href_d2  <= 1'b0;
      data_d2  <= 8'h00;
      mon <= 1'b0;
    end
  else
    begin
      pclk_d2 <= pclk; 
      vsync_d2 <= vsync_d1 ;
      href_d2  <= href_d1;
      data_d2  <= data_d1;
      mon <= (vsync_d2^href_d2^pclk_d2^clk_cnt[25])^(^data_d2)^msg_buf_mon;
    end

//pclk̓͊mFpLED_   
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)begin
    clk_cnt <= 28'b0;
    led_0 <= 1'b0; 
    end
  else if (clk_cnt == led0_cnt_1s) //1bo
    begin
    clk_cnt <= 28'b0;             //JEgl0
    led_0 <= ~led_0;
    end
  else
    begin
    clk_cnt <= clk_cnt + 28'd1;  //JEgAbv
    led_0 <= led_0;
    end
   
always @ (posedge clk_25m or negedge rstb )
  if (rstb==1'b0)begin
    clk_25m_cnt <= 28'b0;
    led_1 <= 1'b0; 
    end
  else if (clk_25m_cnt == c25m_cnt_1s) //1bo
    begin
    clk_25m_cnt <= 28'b0;             //JEgl0
    led_1 <= ~led_1;
    end
  else
    begin
    clk_25m_cnt <= clk_25m_cnt + 28'd1;  //JEgAbv
    led_1 <= led_1;
    end
   
always @ (posedge clk_150m or negedge rstb )
  if (rstb==1'b0)begin
    clk_150m_cnt <= 28'b0;
    led_2 <= 1'b0; 
    end
  else if (clk_150m_cnt == c150m_cnt_1s) //1bo
    begin
    clk_150m_cnt <= 28'b0;             //JEgl0
    led_2 <= ~led_2;
    end
  else
    begin
    clk_150m_cnt <= clk_150m_cnt + 28'd1;  //JEgAbv
    led_2 <= led_2;
    end

always @ (posedge xclk or negedge rstb )
  if (rstb==1'b0)begin
    xclk_cnt <= 28'b0;
    led_3 <= 1'b0; 
    end
  else if (xclk_cnt == c10m_cnt_1s) //1bo
    begin
    xclk_cnt <= 28'b0;             //JEgl0
    led_3 <= ~led_3;
    end
  else
    begin
    xclk_cnt <= xclk_cnt + 28'd1;  //JEgAbv
    led_3 <= led_3;
    end
      
camera_if camera_if(   
  .clk(      clk),        
  .rstb(     rstb),  
  .xclk(     xclk_dmy),  
  .pclk(     pclk),           
  .c_vsync(  c_vsync), 
  .href(     href),           
  .in_data(  in_data), 
  .h_c_en(   h_c_en), 
  .v_c(      v_c),
  .h_c(      h_c),             
  .gen_da_en(in_c_da_en),
  .gen_da_r( in_c_da_r), 
  .gen_da_g( in_c_da_g), 
  .gen_da_b( in_c_da_b),       
  .rx_data(  rx_data),     
  .rx_data_en(rx_data_en),     
  .init_n(      init_n),             
  .wr(       wr),
  .rd(       rd),
  .adr(      adr),
  .wr_data(  wr_data),
  .wr_bytes( wr_bytes),
  .rd_data(  rd_data),
  .rd_data_en(rd_data_en),
  .rd_bytes( rd_bytes),
  .tx_fifo_data_en(tx_fifo_data_en),   
  .tx_fifo_data(tx_fifo_data), 
  .wr_line_end(v_wr_line_end),
  .rd_req(   v_rd_req),
  .wea(      wea),     
  .addra(    addra), 
  .dina(     dina),  
  .addrb(    addrb), 
  .doutb(    doutb),
  .stop_n(   stop_n)
);

assign  v_wr_hadr ={2'b00, addra[9:0]};   
assign  v_wr_vadr ={2'b00, addra[19:10]};

assign  v_rd_hadr ={2'b00, addrb[9:0]};   
assign  v_rd_vadr ={2'b00, addrb[19:10]};
   
assign ram_dq_l = (out_en==1'b1)?ram_dq_w[7:0]:8'bzzzzzzzz;
assign ram_dq_u = (out_en==1'b1)?ram_dq_w[15:8]:8'bzzzzzzzz;
assign ram_dq_r = {ram_dq_u,ram_dq_l};

line_buf_in line_buf_in (
.pclk(       pclk),   
.reset_n(    rstb),  
.v_wr(       wea),          
.v_wr_line_end(v_wr_line_end),          
.v_wr_vadr(  v_wr_vadr),  
.v_wr_hadr(  v_wr_hadr),  
.v_wr_data(  dina),
.ram_clk(    clk_150m), 
.u_wreq(     u_wreq),     
.u_wack(     u_wack),     
.u_wr_da_en( u_wr_da_en),    
.u_wadr(     u_wadr),      
.u_wr_da(    u_wr_da)
);

   
line_buf_out line_buf_out (
.clk(        clk),           
.ram_clk(    clk_150m),    
.reset_n(    rstb), 
.v_rd_req(   v_rd_req),
.v_rd_vadr(  v_rd_vadr),  
.v_rd_hadr(  v_rd_hadr),  
.v_rd_data(  doutb),   
.u_rreq(     u_rreq),     
.u_rack(     u_rack),     
.u_radr(     u_radr),        
.u_rd_da_en( u_rd_da_en),    
.u_rd_da(    u_rd_da)       
);

sdram_ctrl u0(
.clk(      clk_150m),            
.reset_n(  rstb),   
.ram_clke( ram_clke),   
.ram_cs_n( ram_cs_n),  
.ram_cas_n(ram_cas_n),    
.ram_ras_n(ram_ras_n), 
.ram_we_n( ram_we_n),  
.ram_dqml( ram_dqml),  
.ram_dqmu( ram_dqmu),  
.ram_ba0(  ram_ba0),
.ram_ba1(  ram_ba1),   
.ram_adr(  ram_adr),   
.ram_dq_w( ram_dq_w),  
.ram_dq_r( ram_dq_r),  
.u_wreq(    u_wreq),      
.u_wack(    u_wack),          
.u_wadr(    u_wadr),          
.u_wr_da(  u_wr_da),        
.u_wr_da_en(  u_wr_da_en),   
.u_rreq(    u_rreq),           
.u_rack(    u_rack),          
.u_radr(    u_radr),          
.u_rd_da(  u_rd_da),         
.u_rd_da_en(  u_rd_da_en),
.out_en(   out_en),
.state(    state)  
);
      
//svga_if
  assign da_r = da_r_tmp[7:4];
  assign da_g = da_g_tmp[7:4];
  assign da_b = da_b_tmp[7:4];
   
layer_a layer_a(
  .clk(    clk),        
  .rstb(     rstb),   
  .h_c_en(h_c_en),
  .v_c(v_c),
  .h_c(h_c),             
  .gen_da_en(in_a_da_en),
  .gen_da_r( in_a_da_r), 
  .gen_da_g( in_a_da_g), 
  .gen_da_b( in_a_da_b) 
);

layer_b layer_b(
  .clk(    clk),        
  .rstb(     rstb),   
  .h_c_en(h_c_en),
  .v_c(v_c),
  .h_c(h_c),             
  .gen_da_en(in_b_da_en),
  .gen_da_r( in_b_da_r), 
  .gen_da_g( in_b_da_g), 
  .gen_da_b( in_b_da_b) 
);

layer_d layer_d (
  .clk(    clk),        
  .rstb(     rstb),   
  .h_c_en(h_c_en),
  .v_c(v_c),
  .h_c(h_c),             
  .gen_da_en(in_d_da_en),
  .gen_da_r( in_d_da_r), 
  .gen_da_g( in_d_da_g), 
  .gen_da_b( in_d_da_b)
);

svga_if svga_if (
  .clk(         clk),           
  .rstb(        rstb),     
  .h_c_en(   h_c_en),     
  .out_hsync( hsync),   
  .out_vsync( vsync),   
  .v_c(         v_c),           
  .h_c(         h_c),
  /*
  .in_a_da_en(  in_a_da_en),      
  .in_a_da_r(   in_a_da_r),       
  .in_a_da_g(   in_a_da_g),       
  .in_a_da_b(   in_a_da_b),
  */
  .in_a_da_en(  in_c_da_en),      
  .in_a_da_r(   in_c_da_r),       
  .in_a_da_g(   in_c_da_g),       
  .in_a_da_b(   in_c_da_b),      
  .in_b_da_en(  in_b_da_en),      
  .in_b_da_r(   in_b_da_r),       
  .in_b_da_g(   in_b_da_g),       
  .in_b_da_b(   in_b_da_b),   
   /*   
  .in_c_da_en(  in_c_da_en),      
  .in_c_da_r(   in_c_da_r),       
  .in_c_da_g(   in_c_da_g),       
  .in_c_da_b(   in_c_da_b),    
  */   
  .in_c_da_en(  in_a_da_en),      
  .in_c_da_r(   in_a_da_r),       
  .in_c_da_g(   in_a_da_g),       
  .in_c_da_b(   in_a_da_b),      
  .in_d_da_en(  in_d_da_en),      
  .in_d_da_r(   in_d_da_r),       
  .in_d_da_g(   in_d_da_g),       
  .in_d_da_b(   in_d_da_b),       
  .out_da_r(    da_r_tmp),      
  .out_da_g(    da_g_tmp),      
  .out_da_b(    da_b_tmp),      
  .out_da_en(   da_en)        
);
   
// i2c IO driver      
assign sda = (sda_o==1'b0)?1'b0:1'bz;
assign sda_i = sda;
assign scl = (scl_drv==1'b0)?1'b0:1'bz;
   
// i2c master    
i2c_m_if i2c_m_if(
  .clk(   clk),
  .rstb(  rstb),
  .scl(  scl_drv),
  .sda_o(  sda_o),
  .sda_i(  sda_i),
  .wr(    wr),
  .rd(    rd),
  .adr(   adr),
  .wr_data(wr_data),
  .wr_bytes(wr_bytes),
  .rd_data(rd_data),
  .rd_data_en(rd_data_en),
  .rd_bytes(rd_bytes),
  .busy(busy) 
  );
     
rs232c_tx_rx  rs232c_tx_rx(
.RESETB(     rstb),      
.CLK(        clk),           
.TXD(        txd),           
.RXD(        rxd),           
.TX_DATA(    tx_data ),     
.TX_DATA_EN( tx_data_en),  
.TX_BUSY(    tx_busy),     
.RX_DATA(    rx_data),     
.RX_DATA_EN( rx_data_en),  
.RX_BUSY(    rx_busy))             
;
    
msg_buf msg_buf
( 
.CLK(       clk),            
.RESET_N(   rstb),     
.A_DAT_EN(  rx_data_en),  
.A_DAT(     rx_data),     
.A_N_FULL(  a_n_full),     
.B_DAT_EN(  tx_fifo_data_en),  
.B_DAT(     tx_fifo_data), 
.B_N_FULL(  b_n_full),        
.O_DAT(     tx_data),     
.O_DAT_EN(  tx_data_en),
.O_BUSY(    tx_busy),
.MON(       msg_buf_mon)
);
 
dmy_camera dmy_camera(
.xclk   ( xclk  ),
.rstb   ( rstb  ),
.pclk   ( dmy_pclk  ),    
.c_vsync( dmy_c_vsync), 
.href   ( dmy_href  ),    
.in_data( dmy_in_data  ) 
);

assign pclk = (dmy_camera_mode==1'b0)? i_pclk:dmy_pclk;
assign c_vsync = (dmy_camera_mode==1'b0)?i_c_vsync:dmy_c_vsync;
assign href =(dmy_camera_mode==1'b0)?i_href:dmy_href;
assign in_data =(dmy_camera_mode==1'b0)?i_in_data:dmy_in_data;

//mac
   
assign led_tmp = 8'b000000 ;
      
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)begin
    //led_2 <= 1'b0;
    //led_3 <= 1'b0;
    led_4 <= 1'b0;
    led_5 <= 1'b0;
    led_6 <= 1'b0;
    led_7 <= 1'b0;
    led_8 <= 1'b0;
    led_9 <= 1'b0;
  end
  else
    begin
        //led_2 <= led_tmp[0];
        //led_3 <= led_tmp[1];
        led_4 <= led_tmp[2];
        led_5 <= led_tmp[3];
        led_6 <= led_tmp[4];
        led_7 <= led_tmp[5];
        led_8 <= led_tmp[6];
        led_9 <= led_tmp[7];
    end // else: !if(rstb==1'b0)
   

 
endmodule




